# Container is the wrapper for ELF, PE, ...
from miasm2.analysis.binary import Container
# Machine is the wrapper for managing multiple architecture
from miasm2.analysis.machine import Machine

cont = Container.from_stream(open("dump2.bin"))
# Bin stream is a view on the mapped binary
bin_stream = cont.bin_stream

machine = Machine(cont.arch)
# Disassembly engine associated with the current binary
mdis = machine.dis_engine(bin_stream, dont_dis=[0x8049215])
# Disassemble the main function
blocks = mdis.dis_multibloc(cont.entry_point)
# Get back the CFG as a dot file
open("cfg.dot", "w").write(blocks.dot())

from miasm2.core.graph import MatchGraphJoker
# Pattern we want to match:
#           |
#      +----v----+
#      |  (dad)  |
#      |  PUSH   |
#      |  MOV    |
#      +----+----+
#           |
#      +----v----+
#      | (middle)|<---+
#      +----+--+-+    |
#           |  +------+
#      +----v----+
#      |  (end)  |
#      +----+----+
#           |
#           v

# First block must have 2 assembly lines, a PUSH then a MOV, and an unlimited
# number of incomming edge
dad = MatchGraphJoker(name="dad", restrict_in=False,
                      filt=lambda block: (len(block.lines) == 2 and
                                          block.lines[0].name == "PUSH" and
                                          block.lines[1].name == "MOV"))
middle = MatchGraphJoker(name="middle")
# Last block can have an unlimited number of outcomming edge
end = MatchGraphJoker(name="end", restrict_out=False)
# Build a MatchGraph by specifing edges (A >> B means an edge from A to B)
matcher = dad >> middle >> middle >> end

# Visualize it to be sure
open("to_match.dot", "w").write(matcher.dot())


def block_merge(dgs, graph):
    """Remove the pattern defined in matcher
    @dgs: instance of DiGraphSimplifier (for recursive call purpose)
    @graph: graph to simplify
    """
    for sol in matcher.match(graph):
        # /!\ we modify the graph during an iteration on it
        # Our match are disjoint, we can avoid breaking the loop on each match

        # Links each successors to our pattern predecessors
        successors = graph.successors(sol[end])
        for pred in graph.predecessors(sol[dad]):
            for succ in successors:
                graph.add_edge(pred, succ,
                               graph.edges2constraint[(pred, sol[dad])])

        # Remove matched blocks (and associated edges) from the graph
        for node in sol.itervalues():
            graph.del_node(node)

from miasm2.core.graph import DiGraphSimplifier
dgs = DiGraphSimplifier()
dgs.enable_passes([block_merge])

blocks_after = dgs(blocks)
open("cfg_after.dot", "w").write(blocks_after.dot())

key = []
head = blocks_after.heads()[0]
for block in blocks_after.walk_depth_first_forward(head):
    # Depth first traversal starting from the only head
    if (len(block.lines) == 3 and
        block.lines[1].name == "XOR"):
        # Arguments are [EBX + ECX], CST, in Miasm Intermediate Representation
        cst = block.lines[1].args[1]
        key.append(chr(int(cst.arg)))

# Output the resulting key
print "".join(key)
